@@ -111,7 +111,7 @@ test_appstream_parser_legacy ()
 	gchar *path;
 	AsComponent *cpt;
 	GPtrArray *screenshots;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	metad = as_metadata_new ();
 
@@ -144,14 +144,13 @@ test_appstream_parser_locale ()
 {
 	g_autoptr(AsMetadata) metad = NULL;
 	g_autoptr(GFile) file = NULL;
+	g_autoptr(GError) error = NULL;
 	gchar *path;
 	AsComponent *cpt;
 
 	GPtrArray *trs;
 	AsTranslation *tr;
 
-	GError *error = NULL;
-
 	metad = as_metadata_new ();
 
 	path = g_build_filename (datadir, "appdata.xml", NULL);
@@ -210,7 +209,7 @@ test_appstream_write_locale ()
 	GFile *file;
 	gchar *tmp;
 	AsComponent *cpt;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	const gchar *EXPECTED_XML = "<component type=\"desktop-application\">\n"
 				    "  <id>org.mozilla.firefox</id>\n"
@@ -283,6 +282,7 @@ test_appstream_write_description ()
 	guint i;
 	gchar *tmp;
 	AsRelease *rel;
+	g_autoptr(GError) error = NULL;
 	g_autoptr(AsMetadata) metad = NULL;
 	g_autoptr(AsComponent) cpt = NULL;
 
@@ -391,6 +391,7 @@ test_appstream_write_description ()
 					   "</components>\n";
 
 	metad = as_metadata_new ();
+	as_metadata_set_locale (metad, "ALL");
 
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
@@ -437,10 +438,30 @@ test_appstream_write_description ()
 				"<p>Erster paragraph</p>\n<ol><li>Eins</li><li>Zwei</li><li>Drei</li></ol><p>Zweiter Paragraph</p><ul><li>Erstens</li><li>Zweitens</li></ul><p>Paragraph3</p>",
 				"de");
 
+	/* test localization */
 	tmp = as_metadata_component_to_metainfo (metad, AS_FORMAT_KIND_XML, NULL);
 	g_assert (as_xml_test_compare_xml (tmp, EXPECTED_XML_LOCALIZED));
 	g_free (tmp);
 
+	/* test collection-xml conversion */
+	tmp = as_metadata_components_to_collection (metad, AS_FORMAT_KIND_XML, NULL);
+	g_assert (as_xml_test_compare_xml (tmp, EXPECTED_XML_DISTRO));
+	g_free (tmp);
+
+	/* test collection XMl -> metainfo XML */
+	as_metadata_clear_components (metad);
+	as_metadata_set_format_style (metad, AS_FORMAT_STYLE_COLLECTION);
+	as_metadata_parse (metad, EXPECTED_XML_DISTRO, AS_FORMAT_KIND_XML, &error);
+	g_assert_no_error (error);
+	tmp = as_metadata_component_to_metainfo (metad, AS_FORMAT_KIND_XML, NULL);
+	g_assert (as_xml_test_compare_xml (tmp, EXPECTED_XML_LOCALIZED));
+	g_free (tmp);
+
+	/* test metainfo XMl -> collection XML */
+	as_metadata_clear_components (metad);
+	as_metadata_set_format_style (metad, AS_FORMAT_STYLE_METAINFO);
+	as_metadata_parse (metad, EXPECTED_XML_LOCALIZED, AS_FORMAT_KIND_XML, &error);
+	g_assert_no_error (error);
 	tmp = as_metadata_components_to_collection (metad, AS_FORMAT_KIND_XML, NULL);
 	g_assert (as_xml_test_compare_xml (tmp, EXPECTED_XML_DISTRO));
 	g_free (tmp);
